import org.junit.Test;

import static org.junit.Assert.*;

import org.antlr.v4.runtime.tree.*;
import org.antlr.v4.runtime.*;
import org.antlr.v4.runtime.dfa.*;
import org.antlr.v4.runtime.atn.*;
import java.util.BitSet;


public class ArithmeticTest {
    static class E extends BaseErrorListener{
  int errors = 0;
  @Override
  public void reportAmbiguity(Parser recognizer, DFA dfa, int startIndex, int stopIndex, boolean exact, BitSet ambigAlts, ATNConfigSet configs){
//      errors++;
 //     fail("Mehrdeutigkeit entdeckt");
  }
  @Override
  public void reportAttemptingFullContext(Parser recognizer, DFA dfa, int startIndex, int stopIndex, BitSet conflictingAlts, ATNConfigSet configs){
//      errors++;
  //    fail("reportAttemptingFullContext");
  }
  @Override
  public void reportContextSensitivity(Parser recognizer, DFA dfa, int startIndex, int stopIndex, int prediction, ATNConfigSet configs){
//      errors++;
//      fail("Grammatik scheint kontextsensitiv.");
  }
  @Override
  public void syntaxError(Recognizer<?,?> recognizer, Object offendingSymbol, int line, int charPositionInLine, String msg, RecognitionException e){
      errors++;
//      fail("Grammatik kann syntaktisch nicht richtig parsen: "+offendingSymbol+" "+ msg);
  }
    }

  static E getError(String in){
    var input = CharStreams.fromString(in); // a character stream
    var e = new E();
    var lex = new ArithmeticLexer(input); // transforms characters into tokens
    
    lex.removeErrorListeners();
    lex.addErrorListener(e);
    
    CommonTokenStream tokens = new CommonTokenStream(lex); // a token stream
    var parser = new ArithmeticParser(tokens); // transforms tokens into parse trees
    parser.removeErrorListeners();
    parser.addErrorListener(e);
    ParseTree t = parser.start(); // creates the parse tree from the called rule
    return e;
  }

  @Test
  public void test01() {
    successTest("a");
  }
  @Test
  public void test02() {
    successTest("aaaaaaaaa+2");
  }
  @Test
  public void test03() {
    successTest("aaaabbb+x*2");
  }
  @Test
  public void test04() {
    successTest("(aab)");
  }
  @Test
  public void test05() {
    successTest("(17+4)*2");
  }
  @Test
  public void test06() {
    successTest("17+4*2==42");
  }
  @Test
  public void test07() {
    successTest("++++-(x+-4)*2");
  }
  @Test
  public void test08() {
    successTest("x+y*42^15> 17+4*2");
  }
  @Test
  public void test09() {
    successTest("((((++++4%2))))==1+1+1+1+1+1+1+1+1+1*5");
  }

  void successTest(String s) {
    var e = getError(s);
    assertTrue("Fehler beim Parsen von: \""+s+"\""  ,0==e.errors);
 }

  void failTest(String s) {
    var e = getError(s);
    assertTrue("Die Grammatik parst fehlerhaft Wort, das nicht in Sprache ist: \""+s+"\"" ,0!=e.errors);
  }

  @Test
  public void testE01() {
    failTest("ab-");
  }
  @Test
  public void testE02() {
    failTest("b(");
  }
  @Test
  public void testE03() {
    failTest("((aaaabbba)");
  }
  @Test
  public void testE04() {
    failTest("aaaaaaaabbbbbbbb^");
  }
  @Test
  public void testE05() {
    failTest("baaaaaaaa=");
  }
  @Test
  public void testE06() {
    failTest("");
  }
  @Test
  public void testE07() {
    failTest("x>y<z");
  }
  @Test
  public void testE08() {
    failTest("1k+1");
  }
  @Test
  public void testE09() {
    failTest("18+/+19");
  }
}